@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,14 +25,15 @@ from __future__ import print_function
 import traceback
 import os
 import datetime
-import time
 import subprocess
 import gc
+import csv
+
 
 from locale import getpreferredencoding
 from sys import stdout
 
-from common import Debug, isPython3, unicode
+from common import Debug
 
 try:
     STDOUTENCODING = stdout.encoding
@@ -64,11 +65,12 @@ def callers(count=5, exclude=None):
     excluding.extend(['_startRunCallbacks', '_runCallbacks', 'remote_move', 'exec_move'])
     excluding.extend(['proto_message', '_recvMessage', 'remoteMessageReceived'])
     excluding.extend(['clientAction', 'myAction', 'expressionReceived'])
+    excluding.extend(['_read', 'callWithLogger'])
     excluding.extend(['callbackIfDone', 'callback', '__gotAnswer'])
     excluding.extend(['callExpressionReceived', 'proto_answer'])
     excluding.extend(['_dataReceived', 'dataReceived', 'gotItem'])
     excluding.extend(['callWithContext', '_doReadOrWrite', 'doRead'])
-    excluding.extend(['callers', 'debug'])
+    excluding.extend(['callers', 'debug', 'logMessage', 'logDebug'])
     _ = list(([x[2] for x in stck if x[2] not in excluding]))
     names = reversed(_[-count:])
     result = '.'.join(names)
@@ -138,35 +140,7 @@ def get_all_objects():
     return olist
 
 
-def kprint(*args, **kwargs):
-    """
-    A wrapper around print, always encoding unicode to something
-    sensible for the konsole.
-
-    @param args: anything
-    @param kwargs: anything
-    """
-    newArgs = []
-    for arg in args:
-        try:
-            arg = arg.decode('utf-8')
-        except BaseException:
-            arg = repr(arg)
-        arg = arg.encode(STDOUTENCODING, 'ignore')
-        newArgs.append(arg)
-    try:
-        print(*newArgs, sep=kwargs.get('sep', ' '),
-              end=kwargs.get('end', '\n'), file=kwargs.get('file'))
-    except IOError as exception:
-        # very big konsole, busy system: sometimes Python says
-        # resource temporarily not available
-        time.sleep(0.1)
-        print(exception)
-        print(*newArgs, sep=kwargs.get('sep', ' '),
-              end=kwargs.get('end', '\n'), file=kwargs.get('file'))
-
-
-class Duration(object):
+class Duration:
 
     """a helper class for checking code execution duration"""
 
@@ -239,39 +213,63 @@ def gitHead():
     if not os.path.exists(os.path.join('..', '.git')):
         return None
     subprocess.Popen(['git', 'update-index', '-q', '--refresh'])
-    _ = subprocess.Popen(
-        ['git',
-         'diff-index',
-         '--name-only',
-         'HEAD',
-         '--'],
-        stdout=subprocess.PIPE).communicate()[0]
-    uncommitted = list(x.strip() for x in _.split(b'\n') if len(x.strip()))
+    uncommitted = list(popenReadlines('git diff-index --name-only HEAD --'))
     if uncommitted:
         return 'current'
-    result = subprocess.Popen(['git', 'log', '-1', '--format="%h"'],
-                              stdout=subprocess.PIPE).communicate()[0]
-    return result.split(b'\n')[0].replace(b'"', b'')[:15].decode()
+    else:
+        return next(popenReadlines('git log -1 --format=%h'))
 
 
 def xToUtf8(msg, args=None):
     """makes sure msg and all args are utf-8"""
-    if isPython3:
-        if args is not None:
-            return msg, args
-        else:
-            return msg
-    if isinstance(msg, unicode):
-        msg = msg.encode('utf-8')
-    elif not isinstance(msg, str):
-        msg = unicode(msg).encode('utf-8')
     if args is not None:
-        args = list(args[:])
-        for idx, arg in enumerate(args):
-            if isinstance(arg, unicode):
-                args[idx] = arg.encode('utf-8')
-            elif not isinstance(arg, str):
-                args[idx] = unicode(arg).encode('utf-8')
         return msg, args
     else:
         return msg
+    if isinstance(msg, str):
+        msg = msg.encode('utf-8')
+    elif not isinstance(msg, bytes):
+        msg = str(msg).encode('utf-8')
+    if args is not None:
+        args = list(args[:])
+        for idx, arg in enumerate(args):
+            if isinstance(arg, str):
+                args[idx] = arg.encode('utf-8')
+            elif not isinstance(arg, bytes):
+                args[idx] = str(arg).encode('utf-8')
+        return msg, args
+    else:
+        return msg
+
+
+class CsvWriter:
+    """hide differences between Python 2 and 3"""
+    def __init__(self, filename, mode='w'):
+        self.outfile = open(filename, mode)
+        self.__writer = csv.writer(self.outfile, delimiter=Csv.delimiter)
+
+    def writerow(self, row):
+        """write one row"""
+        self.__writer.writerow(list(
+            cell if isinstance(cell, str) else str(cell) for cell in row))
+
+    def __del__(self):
+        """clean up"""
+        self.outfile.close()
+
+class Csv:
+    """hide differences between Python 2 and 3"""
+
+    delimiter = ';'
+
+    @staticmethod
+    def reader(filename):
+        """returns a generator for decoded strings"""
+        return csv.reader(open(filename, 'r', encoding='utf-8'), delimiter=Csv.delimiter)
+
+def popenReadlines(args):
+    """runs a subprocess and returns stdout as a list of unicode encodes lines"""
+    if isinstance(args, str):
+        args = args.split()
+    result = subprocess.Popen(args, universal_newlines=True, stdout=subprocess.PIPE).communicate()[0]
+    return (x.strip() for x in result.split('\n') if x.strip())
